/*
 * This file is part of Siril, an astronomy image processor.
 * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
 * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
 * Reference site is https://free-astro.org/index.php/Siril
 *
 * Siril is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Siril is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Siril. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SRC_ALGOS_SIRIL_WCS_H_
#define SRC_ALGOS_SIRIL_WCS_H_

gboolean has_wcs(fits *fit);
gboolean has_wcsdata(fits *fit);
void free_wcs(fits *fit, gboolean keep_RADEC);
gboolean load_WCS_from_file(fits* fit);
gboolean load_WCS_from_memory(fits *fit);
#ifdef HAVE_WCSLIB
// this one directly uses the WCSLIB struct
void pix2wcs2(struct wcsprm *wcslib, double x, double y, double *r, double *d);
#endif
void pix2wcs(fits *fit, double pixel_x, double pixel_y, double *world_x, double *world_y);
int wcs2pix(fits *fit, double world_x, double world_y, double *pixel_x, double *pixel_y);
void center2wcs(fits *fit, double *r, double *d);
double get_wcs_image_resolution(fits *fit);

#endif /* SRC_ALGOS_SIRIL_WCS_H_ */
